﻿using System;
using System.IO;

namespace TftpServer
{
    internal class BlockReader : IDisposable
    {
        private readonly FileStream _fileStream;

        public string Path { get; }
        public DateTime LastActive { get; private set; }

        public BlockReader(string path)
        {
            Path = path;
            _fileStream = File.OpenRead(path);
            LastActive = DateTime.Now;
        }

        /// <param name="blockNumber">Start from 1</param>
        /// <param name="buffer">Length is block size</param>
        /// <returns>Bytes read</returns>
        public int ReadBlock(int blockNumber, byte[] buffer)
        {
            LastActive = DateTime.Now;
            _fileStream.Position = (blockNumber - 1) * buffer.Length;
            return _fileStream.Read(buffer, 0, buffer.Length);
        }

        public void Dispose()
        {
            _fileStream?.Dispose();
        }
    }
}